@@ -6,21 +6,22 @@
 
 #include <libgnomedb/libgnomedb.h>
 
-extern GdaDict *demo_dict;
+extern GdaConnection *demo_cnc;
+extern GdaSqlParser *demo_parser;
 static GtkWidget *window = NULL;
 
 GtkWidget *
 do_form_pict (GtkWidget *do_widget)
 {  
 	if (!window) {
-                GdaQuery *query;
+		GdaStatement *stmt;
 		GtkWidget *vbox;
 		GtkWidget *label;
 		GdaDataModel *model;
 		GtkWidget *form;
 		GnomeDbRawForm *raw_form;
-		GdaParameterList *data_set;
-		GdaParameter *param;
+		GdaSet *data_set;
+		GdaHolder *param;
 		
 		window = gtk_dialog_new_with_buttons ("Form with the 'picture' plugin",
 						      GTK_WINDOW (do_widget),
@@ -45,30 +46,23 @@ do_form_pict (GtkWidget *do_widget)
 		gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
 		
 		/* Create the demo widget */
-                query = gda_query_new_from_sql (demo_dict, "SELECT id, pict FROM pictures", NULL);
-		model = gda_data_model_query_new (query);
-		g_object_unref (query);
-
-		gda_data_model_query_set_modification_query (GDA_DATA_MODEL_QUERY (model),
-							     "UPDATE pictures set "
-							     "pict=##/*name:'+1' type:'GdaBinary'*/ "
-							     "WHERE id=##/*name:'-0' type:gint*/", NULL);
-		gda_data_model_query_set_modification_query (GDA_DATA_MODEL_QUERY (model),
-							     "DELETE FROM pictures WHERE id=##/*name:'-0' type:gint*/", NULL);
-		gda_data_model_query_set_modification_query (GDA_DATA_MODEL_QUERY (model),
-							     "INSERT INTO pictures (pict) "
-							     "VALUES (##/*name:'+1' type:'GdaBinary'*/)", NULL);
-
+		stmt = gda_sql_parser_parse_string (demo_parser, "SELECT id, pict FROM pictures", NULL, NULL);
+		model = gda_data_model_query_new (demo_cnc, stmt);
+		g_object_unref (stmt);
+		gda_data_model_query_compute_modification_queries (GDA_DATA_MODEL_QUERY (model), NULL,
+								   0, NULL);
 		form = gnome_db_form_new (model);
 		g_object_unref (model);
 
 		/* specify that we want to use the 'picture' plugin */
 		g_object_get (G_OBJECT (form), "raw_form", &raw_form, NULL);
 		data_set = gnome_db_basic_form_get_data_set (GNOME_DB_BASIC_FORM (raw_form));
-		param = gda_parameter_list_find_param (data_set, "pict");
-		g_object_set (G_OBJECT (param), "entry_plugin", "picture", NULL);
+		param = gda_set_get_holder (data_set, "pict");
+		g_object_set (G_OBJECT (param), "plugin", "picture", NULL);
 
 		gtk_box_pack_start (GTK_BOX (vbox), form, TRUE, TRUE, 0);
+
+		gtk_widget_set_size_request (window, 500, 500);
 	}
 
 	if (!GTK_WIDGET_VISIBLE (window))
