@@ -1,6 +1,6 @@
 /* gnome-db-canvas-tip.h
  *
- * Copyright (C) 2002 - 2004 Vivien Malerba
+ * Copyright (C) 2007 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,14 +21,14 @@
 #ifndef __GNOME_DB_CANVAS_TIP__
 #define __GNOME_DB_CANVAS_TIP__
 
-#include <libgnomecanvas/libgnomecanvas.h>
+#include <goocanvas.h>
 
 G_BEGIN_DECLS
 
 
 /*
  * 
- * "Tooltip item" GnomeCanvas item: a CanvasGroup item which is used to present
+ * "Tooltip item" GooCanvas item: a CanvasGroup item which is used to present
  * a tip when the mouse does not move for a time (handled by the CanvasBase object)
  *
  */
@@ -47,7 +47,7 @@ typedef struct _GnomeDbCanvasTipPrivate GnomeDbCanvasTipPrivate;
 /* struct for the object's data */
 struct _GnomeDbCanvasTip
 {
-	GnomeCanvasGroup    object;
+	GooCanvasGroup        object;
 
 	GnomeDbCanvasTipPrivate *priv;
 };
@@ -55,10 +55,11 @@ struct _GnomeDbCanvasTip
 /* struct for the object's class */
 struct _GnomeDbCanvasTipClass
 {
-	GnomeCanvasGroupClass parent_class;
+	GooCanvasGroupClass   parent_class;
 };
 
-GType      gnome_db_canvas_tip_get_type (void);
+GType          gnome_db_canvas_tip_get_type (void) G_GNUC_CONST;
+GooCanvasItem *gnome_db_canvas_tip_new      (GooCanvasItem *parent, const gchar *txt, gdouble x, gdouble y, ...);
 	
 G_END_DECLS
 
