@@ -1,6 +1,6 @@
 /* gnome-db-entry-combo.c
  *
- * Copyright (C) 2003 - 2006 Vivien Malerba
+ * Copyright (C) 2003 - 2008 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -62,11 +62,11 @@ enum
         PROP_SET_DEFAULT_IF_INVALID
 };
 
-/* ComboNode structures: there is one such structure for each GdaParameter in the GdaParameterListNode being
+/* ComboNode structures: there is one such structure for each GdaHolder in the GdaSetNode being
  * used.
  */
 typedef struct {
-	GdaParameterListNode *node;
+	GdaSetNode *node;
 	const GValue       *value;    /* we don't own the value, since it belongs to a GdaDataModel => don't free it */
 	GValue             *value_orig;
 	GValue             *value_default;
@@ -78,8 +78,8 @@ struct  _GnomeDbEntryComboPriv {
         GtkWidget              *combo_entry;
 	GSList                 *combo_nodes; /* list of ComboNode structures */
 
-	GdaParameterList       *paramlist;
-	GdaParameterListSource *source;	
+	GdaSet       *paramlist;
+	GdaSetSource *source;	
 	
 	gboolean                data_valid;
 	gboolean                null_forced;
@@ -211,29 +211,21 @@ gnome_db_entry_combo_init (GnomeDbEntryCombo *combo)
 	combo->priv->source = NULL;
 }
 
-static void
-paramlist_destroyed_cb (GdaParameterList *paramlist, GnomeDbEntryCombo *combo)
-{
-	g_signal_handlers_disconnect_by_func (paramlist, G_CALLBACK (paramlist_destroyed_cb), combo);
-	combo->priv->paramlist = NULL;
-	combo->priv->source = NULL;
-}
-
 /**
  * gnome_db_entry_combo_new
- * @paramlist: a #GdaParameterList object
- * @source: a #GdaParameterListSource structure, part of @paramlist
+ * @paramlist: a #GdaSet object
+ * @source: a #GdaSetSource structure, part of @paramlist
  *
  * Creates a new #GnomeDbEntryCombo widget. The widget is a combo box which displays a
  * selectable list of items (the items come from the 'source->data_model' data model)
  *
- * The widget allows the value setting of one or more #GdaParameter objects
+ * The widget allows the value setting of one or more #GdaHolder objects
  * (one for each 'source->nodes') while proposing potentially "more readable" choices.
  * 
  * Returns: the new widget
  */
 GtkWidget *
-gnome_db_entry_combo_new (GdaParameterList *paramlist, GdaParameterListSource *source)
+gnome_db_entry_combo_new (GdaSet *paramlist, GdaSetSource *source)
 {
 	GObject *obj;
 
@@ -247,30 +239,28 @@ gnome_db_entry_combo_new (GdaParameterList *paramlist, GdaParameterListSource *s
 /**
  * _gnome_db_entry_combo_construct
  * @combo: a #GnomeDbEntryCombo object to be construced
- * @paramlist: a #GdaParameterList object
- * @source: a #GdaParameterListSource structure, part of @paramlist
+ * @paramlist: a #GdaSet object
+ * @source: a #GdaSetSource structure, part of @paramlist
  *
  * 
  * TODO: This is just a work-around for language bindings. Ideally we would use construction
  * properties instead.
 */
 
-void _gnome_db_entry_combo_construct(GnomeDbEntryCombo* combo, 
-                                     GdaParameterList *paramlist, GdaParameterListSource *source)
+void _gnome_db_entry_combo_construct(GnomeDbEntryCombo* combo, GdaSet *paramlist, GdaSetSource *source)
 {
 	GSList *list;
 	GSList *values;
 	GtkWidget *entry;
 	gboolean null_possible;
 
-	g_return_if_fail (GDA_IS_PARAMETER_LIST (paramlist));
+	g_return_if_fail (GDA_IS_SET (paramlist));
 	g_return_if_fail (source);
 	g_return_if_fail (g_slist_find (paramlist->sources_list, source));
   
-  combo->priv->paramlist = paramlist;
+	combo->priv->paramlist = paramlist;
 	combo->priv->source = source;
-	gda_object_connect_destroy (GDA_OBJECT (paramlist), 
-				    G_CALLBACK (paramlist_destroyed_cb), G_OBJECT(combo));	
+	g_object_ref (G_OBJECT (paramlist));
 
 	/* create the ComboNode structures, 
 	 * and use the values provided by the parameters to display the correct row */
@@ -280,12 +270,12 @@ void _gnome_db_entry_combo_construct(GnomeDbEntryCombo* combo,
 	while (list) {
 		ComboNode *cnode = g_new0 (ComboNode, 1);
 		
-		cnode->node = GDA_PARAMETER_LIST_NODE (list->data);
-		cnode->value = gda_parameter_get_value (cnode->node->param);
+		cnode->node = GDA_SET_NODE (list->data);
+		cnode->value = gda_holder_get_value (cnode->node->holder);
 		combo->priv->combo_nodes = g_slist_append (combo->priv->combo_nodes, cnode);
 
 		values = g_slist_append (values, (GValue *) cnode->value);
-		if (gda_parameter_get_not_null (cnode->node->param))
+		if (gda_holder_get_not_null (cnode->node->holder))
 			null_possible = FALSE;
 
 		list = g_slist_next (list);
@@ -330,7 +320,7 @@ gnome_db_entry_combo_dispose (GObject *object)
 
 	if (combo->priv) {
 		if (combo->priv->paramlist) 
-			paramlist_destroyed_cb (combo->priv->paramlist, combo);
+			g_object_unref (combo->priv->paramlist);
 
 		if (combo->priv->combo_nodes) {
 			GSList *list= combo->priv->combo_nodes;
@@ -576,7 +566,7 @@ gnome_db_entry_combo_get_all_values (GnomeDbEntryCombo *combo)
 	g_return_val_if_fail (combo && GNOME_DB_IS_ENTRY_COMBO (combo), NULL);
 	g_return_val_if_fail (combo->priv, NULL);
 
-	return gnome_db_combo_get_values_ext (combo->priv->combo_entry, 0, NULL);
+	return gnome_db_combo_get_values_ext (GNOME_DB_COMBO (combo->priv->combo_entry), 0, NULL);
 }
 
 /**
