@@ -1,6 +1,6 @@
 /* gnome-db-entry-string.c
  *
- * Copyright (C) 2003 - 2007 Vivien Malerba
+ * Copyright (C) 2003 - 2008 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -74,11 +74,17 @@ static void set_entry_options (GnomeDbEntryString *mgstr, const gchar *options);
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass  *parent_class = NULL;
 
+typedef enum {
+	TYPE_UNKNOWN, /* when value is undefined */
+	TYPE_NUMERIC,
+	TYPE_NOT_NUMERIC
+} InternalType;
+
 /* private structure */
 struct _GnomeDbEntryStringPrivate
 {
 	gboolean       multiline;
-	gboolean       is_numeric_type;
+	InternalType   internal_type;
 	GtkWidget     *vbox;
 
 	GtkWidget     *entry;
@@ -106,7 +112,7 @@ gnome_db_entry_string_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbEntryStringClass),
 			(GBaseInitFunc) NULL,
@@ -173,7 +179,7 @@ gnome_db_entry_string_init (GnomeDbEntryString * mgstr)
 	mgstr->priv->view = NULL;
 	mgstr->priv->sw = NULL;
 
-	mgstr->priv->is_numeric_type = TRUE;
+	mgstr->priv->internal_type = TYPE_UNKNOWN;
 	mgstr->priv->maxsize = 0; /* unlimited */
 	mgstr->priv->thousand_sep = 0;
 	mgstr->priv->nb_decimals = -1; /* unlimited number of decimals */
@@ -182,6 +188,18 @@ gnome_db_entry_string_init (GnomeDbEntryString * mgstr)
 	mgstr->priv->entry_change_sig = 0;
 }
 
+static InternalType
+determine_internal_type (GType type)
+{
+	if ((type == G_TYPE_INT64) || (type == G_TYPE_UINT64) || (type == G_TYPE_DOUBLE) ||
+	    (type == G_TYPE_INT) || (type == GDA_TYPE_NUMERIC) || (type == G_TYPE_FLOAT) ||
+	    (type == GDA_TYPE_SHORT) || (type == GDA_TYPE_USHORT) || (type == G_TYPE_CHAR) ||
+	    (type == G_TYPE_UCHAR) || (type == G_TYPE_LONG) || (type ==  G_TYPE_ULONG) || (type == G_TYPE_UINT))
+		return TYPE_NUMERIC;
+	else
+		return TYPE_NOT_NUMERIC;
+}
+
 /**
  * gnome_db_entry_string_new
  * @dh: the data handler to be used by the new widget
@@ -205,12 +223,7 @@ gnome_db_entry_string_new (GdaDataHandler *dh, GType type, const gchar *options)
 	mgstr = GNOME_DB_ENTRY_STRING (obj);
 	gnome_db_data_entry_set_value_type (GNOME_DB_DATA_ENTRY (mgstr), type);
 
-	mgstr->priv->is_numeric_type = FALSE;
-	if ((type == G_TYPE_INT64) || (type == G_TYPE_UINT64) || (type == G_TYPE_DOUBLE) ||
-	    (type == G_TYPE_INT) || (type == GDA_TYPE_NUMERIC) || (type == G_TYPE_FLOAT) ||
-	    (type == GDA_TYPE_SHORT) || (type == GDA_TYPE_USHORT) || (type == G_TYPE_CHAR) ||
-	    (type == G_TYPE_UCHAR) || (type == G_TYPE_LONG) || (type ==  G_TYPE_ULONG) || (type == G_TYPE_UINT))
-		mgstr->priv->is_numeric_type = TRUE;
+	mgstr->priv->internal_type = determine_internal_type (type);
 	g_object_set (obj, "options", options, NULL);
 
 	return GTK_WIDGET (obj);
@@ -275,7 +288,12 @@ gnome_db_entry_string_set_property (GObject *object,
 	if (mgstr->priv) {
 		switch (param_id) {
 		case PROP_MULTILINE:
-			if (mgstr->priv->is_numeric_type)
+			if (mgstr->priv->internal_type == TYPE_UNKNOWN)
+				mgstr->priv->internal_type = 
+					determine_internal_type (gnome_db_data_entry_get_value_type 
+								 (GNOME_DB_DATA_ENTRY (mgstr)));
+
+			if (mgstr->priv->internal_type == TYPE_NUMERIC)
 				return;
 
 			if (g_value_get_boolean (value) != mgstr->priv->multiline) {
@@ -387,6 +405,8 @@ real_set_value (GnomeDbEntryWrapper *mgwrap, const GValue *value)
 
 	PangoLayout *layout;
 	gchar *text;
+	static gchar *too_long_msg = NULL;
+	static gint too_long_msg_len;
 	
 	g_return_if_fail (GNOME_DB_IS_ENTRY_STRING (mgwrap));
 	mgstr = GNOME_DB_ENTRY_STRING (mgwrap);
@@ -395,58 +415,43 @@ real_set_value (GnomeDbEntryWrapper *mgwrap, const GValue *value)
 	dh = gnome_db_data_entry_get_handler (GNOME_DB_DATA_ENTRY (mgwrap));
 
 	/* do we need to go into multi line mode ? */
+	if (!too_long_msg) {
+		too_long_msg = _("<string cut because too long>");
+		too_long_msg_len = strlen (too_long_msg);
+	}
 	text = gda_data_handler_get_str_from_value (dh, value);
+	if (text && (strlen (text) > MAX_ACCEPTED_STRING_LENGTH + too_long_msg_len)) {
+		memmove (text + too_long_msg_len, text, MAX_ACCEPTED_STRING_LENGTH);
+		memcpy (text, too_long_msg, too_long_msg_len);
+		text [MAX_ACCEPTED_STRING_LENGTH + too_long_msg_len + 1] = 0;
+	}
+
 	layout = gtk_widget_create_pango_layout (GTK_WIDGET (mgwrap), text);
-	g_free (text);
 	if (pango_layout_get_line_count (layout) > 1) 
 		g_object_set (G_OBJECT (mgwrap), "multiline", TRUE, NULL);
 	g_object_unref (G_OBJECT (layout));
-
+	
 	/* fill the single line widget */
 	if (value) {
 		if (gda_value_is_null ((GValue *) value))
 			gnome_db_format_entry_set_text (GNOME_DB_FORMAT_ENTRY (mgstr->priv->entry), NULL);
-		else {
-			gchar *str;
-
-			str = gda_data_handler_get_str_from_value (dh, value);
-			if (str) {
-				if (strlen (str) > MAX_ACCEPTED_STRING_LENGTH) {
-					gchar *msg;
-					gint len;
-					msg = _("<string cut because too long>");
-					len = strlen (msg);
-					str [MAX_ACCEPTED_STRING_LENGTH] = 0;
-					memmove (str + len, str, MAX_ACCEPTED_STRING_LENGTH - len);
-					memcpy (str, msg, strlen (msg));
-				}
-				gnome_db_format_entry_set_text (GNOME_DB_FORMAT_ENTRY (mgstr->priv->entry), str);
-				g_free (str);
-			}
-			else
-				gnome_db_format_entry_set_text (GNOME_DB_FORMAT_ENTRY (mgstr->priv->entry), NULL);
-		}
+		else 
+			gnome_db_format_entry_set_text (GNOME_DB_FORMAT_ENTRY (mgstr->priv->entry), text);
 	}
-	else 
+	else
 		gnome_db_format_entry_set_text (GNOME_DB_FORMAT_ENTRY (mgstr->priv->entry), NULL);
 
 	/* fill the multiline widget */
 	if (value) {
-		if (gda_value_is_null ((GValue *) value))
+		if (gda_value_is_null ((GValue *) value) || !text)
                         gtk_text_buffer_set_text (mgstr->priv->buffer, "", -1);
-		else {
-			gchar *str;
-
-			str = gda_data_handler_get_str_from_value (dh, value);
-			if (str) {
-				gtk_text_buffer_set_text (mgstr->priv->buffer, str, -1);
-				g_free (str);
-			}
-		}
+		else 
+			gtk_text_buffer_set_text (mgstr->priv->buffer, text, -1);
 	}
 	else 
 		gtk_text_buffer_set_text (mgstr->priv->buffer, "", -1);
 
+	g_free (text);
 }
 
 static GValue *
@@ -635,8 +640,13 @@ set_entry_options (GnomeDbEntryString *mgstr, const gchar *options)
                 GdaQuarkList *params;
                 const gchar *str;
 		
+		if (mgstr->priv->internal_type == TYPE_UNKNOWN)
+			mgstr->priv->internal_type = 
+				determine_internal_type (gnome_db_data_entry_get_value_type 
+							 (GNOME_DB_DATA_ENTRY (mgstr)));
+
                 params = gda_quark_list_new_from_string (options);
-		if (!mgstr->priv->is_numeric_type) {
+		if (mgstr->priv->internal_type == TYPE_NOT_NUMERIC) {
 			/* STRING specific options */
 			str = gda_quark_list_find (params, "MAX_SIZE");
 			if (str) 
