@@ -1,6 +1,6 @@
 /* gnome-db-data-entry.c
  *
- * Copyright (C) 2003 - 2006 Vivien Malerba
+ * Copyright (C) 2003 - 2007 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -46,10 +46,11 @@ enum
 	CONTENTS_MODIFIED,
 	CONTENTS_ACTIVATED,
 	STATUS_CHANGED,
+	CONTENTS_VALID,
 	LAST_SIGNAL
 };
 
-static gint gnome_db_data_entry_signals[LAST_SIGNAL] = { 0 };
+static gint gnome_db_data_entry_signals[LAST_SIGNAL] = { 0, 0, 0, 0 };
 static void gnome_db_data_entry_iface_init (gpointer g_class);
 
 GType
@@ -57,7 +58,7 @@ gnome_db_data_entry_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbDataEntryIface),
 			(GBaseInitFunc) gnome_db_data_entry_iface_init,
@@ -75,6 +76,25 @@ gnome_db_data_entry_get_type (void)
 	return type;
 }
 
+static gboolean
+contents_valid_accumulator (GSignalInvocationHint *ihint,
+		   GValue *return_accu,
+		   const GValue *handler_return,
+		   gpointer data)
+{
+        gboolean thisvalue;
+
+        thisvalue = g_value_get_boolean (handler_return);
+        g_value_set_boolean (return_accu, thisvalue); 
+
+        return thisvalue; /* stop signal if 'thisvalue' is FALSE */
+}
+
+static gboolean
+m_class_contents_valid (GnomeDbDataEntry *de, GError **error)
+{
+	return TRUE;
+}
 
 static void
 gnome_db_data_entry_iface_init (gpointer g_class)
@@ -88,7 +108,7 @@ gnome_db_data_entry_iface_init (gpointer g_class)
 				      G_SIGNAL_RUN_FIRST,
 				      G_STRUCT_OFFSET (GnomeDbDataEntryIface, contents_modified),
 				      NULL, NULL,
-				      marshal_VOID__VOID,
+				      gnome_db_marshal_VOID__VOID,
 				      G_TYPE_NONE, 0);
 		gnome_db_data_entry_signals[CONTENTS_ACTIVATED] =
 			g_signal_new ("contents_activated",
@@ -96,7 +116,7 @@ gnome_db_data_entry_iface_init (gpointer g_class)
 				      G_SIGNAL_RUN_FIRST,
 				      G_STRUCT_OFFSET (GnomeDbDataEntryIface, contents_activated),
 				      NULL, NULL,
-				      marshal_VOID__VOID,
+				      gnome_db_marshal_VOID__VOID,
 				      G_TYPE_NONE, 0);
 		gnome_db_data_entry_signals[STATUS_CHANGED] =
 			g_signal_new ("status_changed",
@@ -104,9 +124,18 @@ gnome_db_data_entry_iface_init (gpointer g_class)
 				      G_SIGNAL_RUN_FIRST,
 				      G_STRUCT_OFFSET (GnomeDbDataEntryIface, status_changed),
 				      NULL, NULL,
-				      marshal_VOID__VOID,
+				      gnome_db_marshal_VOID__VOID,
 				      G_TYPE_NONE, 0);
+		gnome_db_data_entry_signals[CONTENTS_VALID] =
+			g_signal_new ("contents_valid",
+				      GNOME_DB_TYPE_DATA_ENTRY,
+				      G_SIGNAL_RUN_LAST,
+				      G_STRUCT_OFFSET (GnomeDbDataEntryIface, contents_valid),
+				      contents_valid_accumulator, NULL,
+				      gnome_db_marshal_BOOLEAN__POINTER,
+				      G_TYPE_BOOLEAN, 1, G_TYPE_POINTER);
 
+		((GnomeDbDataEntryIface*) g_class)->contents_valid = m_class_contents_valid;
 		initialized = TRUE;
 	}
 }
@@ -148,7 +177,6 @@ gnome_db_data_entry_get_value_type (GnomeDbDataEntry *de)
 	return G_TYPE_INVALID;
 }
 
-
 /**
  * gnome_db_data_entry_set_value
  * @de: a #GtkWidget object which implements the #GnomeDbDataEntry interface
@@ -189,6 +217,29 @@ gnome_db_data_entry_get_value (GnomeDbDataEntry *de)
 	return NULL;
 }
 
+/**
+ * gnome_db_data_entry_is_contents_valid
+ * @de: a #GtkWidget object which implements the #GnomeDbDataEntry interface
+ * @error: a place to store an error, or %NULL
+ *
+ * Tests the validity of @de's contents. The validity is a determined from:
+ * <itemizedlist>
+ * <listitem><para>the @de widget itself if it is capable of doing it (depending on the implementation)</para></listitem>
+ * <listitem><para>the results of the "contents_valid" signal which can be connected from </para></listitem>
+ * </itemizedlist>
+ *
+ * Returns: TRUE if @de's contents is valid
+ */
+gboolean
+gnome_db_data_entry_is_contents_valid (GnomeDbDataEntry *de, GError **error)
+{
+	gboolean is_valid;
+	g_return_val_if_fail (GNOME_DB_IS_DATA_ENTRY (de), FALSE);
+
+	g_signal_emit (de, gnome_db_data_entry_signals [CONTENTS_VALID], 0, error, &is_valid);
+	return is_valid;
+}
+
 
 /**
  * gnome_db_data_entry_set_value_orig
