@@ -169,28 +169,23 @@ data_widget_destroyed_cb (GnomeDbDataWidget *wid, GnomeDbDataWidgetInfo *info)
 
 
 static void
-proxy_destroyed_cb (GdaDataProxy *proxy, GnomeDbDataWidgetInfo *info)
+release_proxy (GnomeDbDataWidgetInfo *info)
 {
-	g_assert (proxy == info->priv->proxy);
-	g_signal_handlers_disconnect_by_func (G_OBJECT (proxy),
-					      G_CALLBACK (proxy_destroyed_cb), info);
-	g_signal_handlers_disconnect_by_func (G_OBJECT (proxy),
+	g_signal_handlers_disconnect_by_func (G_OBJECT (info->priv->proxy),
 					      G_CALLBACK (proxy_changed_cb), info);
-	g_signal_handlers_disconnect_by_func (G_OBJECT (proxy),
+	g_signal_handlers_disconnect_by_func (G_OBJECT (info->priv->proxy),
 					      G_CALLBACK (proxy_sample_changed_cb), info);
-	g_signal_handlers_disconnect_by_func (G_OBJECT (proxy),
+	g_signal_handlers_disconnect_by_func (G_OBJECT (info->priv->proxy),
 					      G_CALLBACK (proxy_row_changed_cb), info);
+	g_object_unref (info->priv->proxy);
 	info->priv->proxy = NULL;
 }
 
 static void iter_row_changed_cb (GdaDataModelIter *iter, gint row, GnomeDbDataWidgetInfo *info);
 static void
-iter_destroyed_cb (GdaDataModelIter *iter, GnomeDbDataWidgetInfo *info)
+release_iter (GnomeDbDataWidgetInfo *info)
 {
-	g_assert (iter == info->priv->iter);
-	g_signal_handlers_disconnect_by_func (iter,
-					      G_CALLBACK (iter_destroyed_cb), info);
-	g_signal_handlers_disconnect_by_func (iter,
+	g_signal_handlers_disconnect_by_func (info->priv->iter,
 					      G_CALLBACK (iter_row_changed_cb), info);
 	info->priv->iter = NULL;
 }
@@ -212,9 +207,9 @@ gnome_db_data_widget_info_dispose (GObject *object)
 
 	if (info->priv) {
 		if (info->priv->proxy)
-			proxy_destroyed_cb (info->priv->proxy, info);
+			release_proxy (info);
 		if (info->priv->iter)
-			iter_destroyed_cb (info->priv->iter, info);
+			release_iter (info);
 		if (info->priv->data_widget)
 			data_widget_destroyed_cb (info->priv->data_widget, info);
 
@@ -242,11 +237,11 @@ gnome_db_data_widget_info_set_property (GObject *object,
 			if (info->priv->data_widget)
 				data_widget_destroyed_cb (info->priv->data_widget, info);
 			if (info->priv->iter) 
-				iter_destroyed_cb (info->priv->iter, info);
+				release_iter (info);
 			if (info->priv->proxy)
-				proxy_destroyed_cb (info->priv->proxy, info);
+				release_proxy (info);
 
-			info->priv->data_widget = GNOME_DB_DATA_WIDGET(g_value_get_object (value));
+			info->priv->data_widget = GNOME_DB_DATA_WIDGET (g_value_get_object (value));
 			if (info->priv->data_widget) {
 				GdaDataProxy *proxy;
 				GdaDataModelIter *iter;
@@ -264,8 +259,7 @@ gnome_db_data_widget_info_set_property (GObject *object,
 				proxy = gnome_db_data_widget_get_proxy (info->priv->data_widget);
 				if (proxy) {
 					info->priv->proxy = proxy;
-					gda_object_connect_destroy (proxy,
-								    G_CALLBACK (proxy_destroyed_cb), info);
+					g_object_ref (info->priv->proxy);
 					g_signal_connect (G_OBJECT (proxy), "changed",
 							  G_CALLBACK (proxy_changed_cb), info);
 					g_signal_connect (G_OBJECT (proxy), "sample_changed",
@@ -280,8 +274,7 @@ gnome_db_data_widget_info_set_property (GObject *object,
 										      (info->priv->data_widget));
 					info->priv->iter = iter;
 					if (iter) {
-						gda_object_connect_destroy (iter,
-									    G_CALLBACK (iter_destroyed_cb), info);
+						g_object_ref (G_OBJECT (iter));
 						g_signal_connect (iter, "row_changed",
 								  G_CALLBACK (iter_row_changed_cb), info);
 					}
