@@ -40,7 +40,7 @@ gnome_db_data_widget_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbDataWidgetIface),
 			(GBaseInitFunc) gnome_db_data_widget_iface_init,
@@ -189,7 +189,8 @@ gnome_db_data_widget_column_show_actions (GnomeDbDataWidget *iface, gint column,
  * <listitem><para>Record by record moving: "ActionFirstRecord", "ActionPrevRecord", 
  *    "ActionNextRecord", "ActionLastRecord",</para></listitem>
  * <listitem><para>Chuncks of records moving: "ActionFirstChunck", "ActionPrevChunck", 
- *     "ActionNextChunck", "ActionLastChunck".</para></listitem></itemizedlist>
+ *     "ActionNextChunck", "ActionLastChunck".</para></listitem>
+ * <listitem><para>Filtering: "ActionFilter"</para></listitem></itemizedlist>
  * 
  * Returns: the #GtkActionGroup with all the possible actions on the widget.
  */
@@ -285,7 +286,7 @@ gnome_db_data_widget_perform_action (GnomeDbDataWidget *iface, GnomeDbAction act
  * data stored in @iface. When the user changes what's displayed or what's selected (depending on the
  * actual widget) in @iface, then the parameter's values change as well.
  *
- * Returns: the #GdaParameterList object for data (not a new object)
+ * Returns: the #GdaSet object for data (not a new object)
  */
 GdaDataModelIter *
 gnome_db_data_widget_get_current_data (GnomeDbDataWidget *iface)
