@@ -1,5 +1,5 @@
 /* GNOME DB library
- * Copyright (C) 1999 - 2007 The GNOME Foundation
+ * Copyright (C) 1999 - 2008 The GNOME Foundation
  *
  * AUTHORS:
  *      Rodrigo Moya <rodrigo@gnome-db.org>
@@ -40,17 +40,17 @@ typedef struct _GnomeDbDsnAssistantClass   GnomeDbDsnAssistantClass;
 typedef struct _GnomeDbDsnAssistantPrivate GnomeDbDsnAssistantPrivate;
 
 struct _GnomeDbDsnAssistant {
-	GtkAssistant                      assistant;
+	GtkAssistant                assistant;
 	GnomeDbDsnAssistantPrivate *priv;
 };
 
 struct _GnomeDbDsnAssistantClass {
-	GtkAssistantClass                 parent_class;
+	GtkAssistantClass           parent_class;
 
-	void                           (* finished) (GnomeDbDsnAssistant *assistant, gboolean error);
+	void                     (* finished) (GnomeDbDsnAssistant *assistant, gboolean error);
 };
 
-GType                    gnome_db_dsn_assistant_get_type (void);
+GType                    gnome_db_dsn_assistant_get_type (void) G_GNUC_CONST;
 
 GtkWidget               *gnome_db_dsn_assistant_new      (void);
 const GdaDataSourceInfo *gnome_db_dsn_assistant_get_dsn  (GnomeDbDsnAssistant *assistant);
