@@ -1,6 +1,6 @@
 /* gnome-db-form.c
  *
- * Copyright (C) 2002 - 2006 Vivien Malerba
+ * Copyright (C) 2002 - 2007 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -53,6 +53,7 @@ enum
         PROP_0,
         PROP_RAW_FORM,
 	PROP_INFO,
+	PROP_MODEL
 };
 
 GType
@@ -60,7 +61,7 @@ gnome_db_form_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbFormClass),
 			(GBaseInitFunc) NULL,
@@ -94,8 +95,13 @@ gnome_db_form_class_init (GnomeDbFormClass *class)
                                          g_param_spec_object ("raw_form", NULL, NULL, GNOME_DB_TYPE_RAW_FORM,
                                                                G_PARAM_READABLE));
 	g_object_class_install_property (object_class, PROP_INFO,
-                                         g_param_spec_object ("widget_info", NULL, NULL, GNOME_DB_TYPE_DATA_WIDGET,
-							                                   G_PARAM_READABLE));
+                                         g_param_spec_object ("widget_info", NULL, NULL, GNOME_DB_TYPE_DATA_WIDGET_INFO,
+							      G_PARAM_READABLE));
+	
+	g_object_class_install_property (object_class, PROP_MODEL,
+					 g_param_spec_object ("model", NULL, NULL,
+							      GDA_TYPE_DATA_MODEL,
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 static void
@@ -111,7 +117,7 @@ gnome_db_form_init (GnomeDbForm *form)
 
 	form->priv->info = gnome_db_data_widget_info_new (GNOME_DB_DATA_WIDGET (form->priv->raw_form), 
 							  GNOME_DB_DATA_WIDGET_INFO_CURRENT_ROW |
-							  GNOME_DB_DATA_WIDGET_INFO_ROW_MODIF_BUTTONS |
+							  GNOME_DB_DATA_WIDGET_INFO_ROW_MODIFY_BUTTONS |
 							  GNOME_DB_DATA_WIDGET_INFO_ROW_MOVE_BUTTONS);
 	gtk_box_pack_start (GTK_BOX (form), form->priv->info, FALSE, TRUE, 0);
 	gtk_widget_show (form->priv->info);
@@ -130,14 +136,12 @@ GtkWidget *
 gnome_db_form_new (GdaDataModel *model)
 {
 	GnomeDbForm *form;
-	GnomeDbRawForm *raw_form;
 	
 	g_return_val_if_fail (!model || GDA_IS_DATA_MODEL (model), NULL);
 	
-	form = (GnomeDbForm *) g_object_new (GNOME_DB_TYPE_FORM, NULL);
+	form = (GnomeDbForm *) g_object_new (GNOME_DB_TYPE_FORM,
+	                                     "model", model, NULL);
 	
-	g_object_set (form->priv->raw_form, "model", model, NULL);
-		
 	return (GtkWidget *) form;
 }
 
@@ -149,11 +153,17 @@ gnome_db_form_set_property (GObject *object,
 				 GParamSpec *pspec)
 {
 	GnomeDbForm *form;
-	
+	GdaDataModel *model;
+
 	form = GNOME_DB_FORM (object);
 	
 	if (form->priv) {
 		switch (param_id) {
+		case PROP_MODEL:
+			model = GDA_DATA_MODEL (g_value_get_object (value));
+			g_object_set (G_OBJECT(form->priv->raw_form),
+			              "model", model, NULL);
+			break;
 			
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
@@ -169,6 +179,7 @@ gnome_db_form_get_property (GObject *object,
 				 GParamSpec *pspec)
 {
 	GnomeDbForm *form;
+	GdaDataModel *model;
 
         form = GNOME_DB_FORM (object);
         if (form->priv) {
@@ -179,6 +190,11 @@ gnome_db_form_get_property (GObject *object,
 		case PROP_INFO:
 			g_value_set_object (value, form->priv->info);
 			break;
+		case PROP_MODEL:
+			g_object_get (G_OBJECT (form->priv->raw_form),
+			              "model", &model, NULL);
+			g_value_take_object (value, G_OBJECT (model));
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 			break;
