@@ -1,6 +1,6 @@
 /* gnome-db-raw-grid.c
  *
- * Copyright (C) 2002 - 2007 Vivien Malerba
+ * Copyright (C) 2002 - 2008 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,6 +24,7 @@
 #include <libgda/libgda.h>
 #include "gnome-db-raw-grid.h"
 #include "gnome-db-data-widget.h"
+#include "gnome-db-data-widget-filter.h"
 #include "utility.h"
 #include "gnome-db-tools.h"
 #include "gnome-db-util.h"
@@ -46,12 +47,11 @@ static void gnome_db_raw_grid_get_property (GObject *object,
 
 static void init_tree_view (GnomeDbRawGrid *grid);
 
-static void proxy_destroyed_cb (GdaDataProxy *proxy, GnomeDbRawGrid *grid);
 static void proxy_sample_changed_cb (GdaDataProxy *proxy, gint sample_start, gint sample_end, GnomeDbRawGrid *grid);
 static void proxy_row_updated_cb (GdaDataProxy *proxy, gint proxy_row, GnomeDbRawGrid *grid);
 static void proxy_reset_cb (GdaDataProxy *proxy, GnomeDbRawGrid *grid);
-static void paramlist_public_data_changed_cb (GdaParameterList *paramlist, GnomeDbRawGrid *grid);
-static void paramlist_param_plugin_changed_cb (GdaParameterList *paramlist, GdaParameter *param, GnomeDbRawGrid *grid);
+static void paramlist_public_data_changed_cb (GdaSet *paramlist, GnomeDbRawGrid *grid);
+static void paramlist_param_plugin_changed_cb (GdaSet *paramlist, GdaHolder *param, GnomeDbRawGrid *grid);
 static gboolean iter_row_to_change_cb (GdaDataModelIter *iter, gint row, GnomeDbRawGrid *grid);
 static void iter_row_changed_cb (GdaDataModelIter *iter, gint row, GnomeDbRawGrid *grid);
 
@@ -70,7 +70,7 @@ static gboolean        gnome_db_raw_grid_widget_set_write_mode (GnomeDbDataWidge
 static GnomeDbDataWidgetWriteMode gnome_db_raw_grid_widget_get_write_mode (GnomeDbDataWidget *iface);
 
 typedef struct {
-	GdaParameterListGroup *group;
+	GdaSetGroup *group;
 	GtkCellRenderer       *data_cell;
 	GtkCellRenderer       *info_cell;
 	gboolean               info_shown;
@@ -79,7 +79,7 @@ typedef struct {
 
 #define COLUMN_DATA(x) ((ColumnData *)(x))
 
-static ColumnData *get_column_data (GnomeDbRawGrid *grid, GdaParameterListGroup *group);
+static ColumnData *get_column_data (GnomeDbRawGrid *grid, GdaSetGroup *group);
 
 struct _GnomeDbRawGridPriv
 {
@@ -97,6 +97,9 @@ struct _GnomeDbRawGridPriv
 
 	gint                        export_type; /* used by the export dialog */
 	GnomeDbDataWidgetWriteMode  write_mode;
+
+	GtkWidget                  *filter;
+	GtkWidget                  *filter_window;
 };
 
 /* get a pointer to the parents to be able to call their destructor */
@@ -146,6 +149,7 @@ static void action_first_chunck_cb (GtkAction *action, GnomeDbRawGrid *grid);
 static void action_prev_chunck_cb (GtkAction *action, GnomeDbRawGrid *grid);
 static void action_next_chunck_cb (GtkAction *action, GnomeDbRawGrid *grid);
 static void action_last_chunck_cb (GtkAction *action, GnomeDbRawGrid *grid);
+static void action_filter_cb (GtkAction *action, GnomeDbRawGrid *grid);
 
 static GtkActionEntry ui_actions[] = {
 	{ "ActionNew", GTK_STOCK_ADD, "_New", NULL, "Create a new data entry", G_CALLBACK (action_new_cb)},
@@ -161,7 +165,9 @@ static GtkActionEntry ui_actions[] = {
 	{ "ActionPrevChunck", GTK_STOCK_GO_BACK, "_Previous chunck", NULL, "Go to previous chunck of records", 
 	  G_CALLBACK (action_prev_chunck_cb)},
 	{ "ActionNextChunck", GTK_STOCK_GO_FORWARD, "Ne_xt chunck", NULL, "Go to next chunck of records",
-	  G_CALLBACK (action_next_chunck_cb)}
+	  G_CALLBACK (action_next_chunck_cb)},
+	{ "ActionFilter", GTK_STOCK_FIND, "Filter", NULL, "Filter records",
+	  G_CALLBACK (action_filter_cb)}
 };
 
 GType
@@ -169,7 +175,7 @@ gnome_db_raw_grid_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbRawGridClass),
 			(GBaseInitFunc) NULL,
@@ -211,10 +217,9 @@ gnome_db_raw_grid_widget_init (GnomeDbDataWidgetIface *iface)
 }
 
 static void
-gnome_db_raw_grid_class_init (GnomeDbRawGridClass * class)
+gnome_db_raw_grid_class_init (GnomeDbRawGridClass *class)
 {
-	GObjectClass   *object_class = G_OBJECT_CLASS (class);
-	
+	GObjectClass  *object_class = G_OBJECT_CLASS (class);
 	parent_class = g_type_class_peek_parent (class);
 
 	gnome_db_raw_grid_signals[SELECTION_CHANGED] = 
@@ -223,7 +228,7 @@ gnome_db_raw_grid_class_init (GnomeDbRawGridClass * class)
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GnomeDbRawGridClass, selection_changed),
                               NULL, NULL,
-                              marshal_VOID__BOOLEAN, G_TYPE_NONE,
+                              gnome_db_marshal_VOID__BOOLEAN, G_TYPE_NONE,
                               1, G_TYPE_BOOLEAN);
 	gnome_db_raw_grid_signals[DOUBLE_CLICKED] = 
 		g_signal_new ("double_clicked",
@@ -231,7 +236,7 @@ gnome_db_raw_grid_class_init (GnomeDbRawGridClass * class)
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GnomeDbRawGridClass, double_clicked),
                               NULL, NULL,
-                              marshal_VOID__INT, G_TYPE_NONE,
+                              gnome_db_marshal_VOID__INT, G_TYPE_NONE,
                               1, G_TYPE_INT);
 	gnome_db_raw_grid_signals[POPULATE_POPUP] = 
 		g_signal_new ("populate_popup",
@@ -239,7 +244,7 @@ gnome_db_raw_grid_class_init (GnomeDbRawGridClass * class)
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GnomeDbRawGridClass, populate_popup),
                               NULL, NULL,
-                              marshal_VOID__OBJECT, G_TYPE_NONE,
+                              gnome_db_marshal_VOID__OBJECT, G_TYPE_NONE,
                               1, GTK_TYPE_MENU);
 
 	object_class->dispose = gnome_db_raw_grid_dispose;
@@ -281,6 +286,7 @@ gnome_db_raw_grid_init (GnomeDbRawGrid *grid)
 			  G_CALLBACK (tree_view_event_cb), grid);
 	g_signal_connect (G_OBJECT (tree_view), "button_press_event",
                           G_CALLBACK (tree_view_popup_button_pressed_cb), grid);
+	gtk_tree_view_set_enable_search (tree_view, FALSE);
 
 	/* selection and signal handling */
 	selection = gtk_tree_view_get_selection (tree_view);
@@ -293,6 +299,9 @@ gnome_db_raw_grid_init (GnomeDbRawGrid *grid)
 	/* action group */
 	grid->priv->actions_group = gtk_action_group_new ("Actions");
 	gtk_action_group_add_actions (grid->priv->actions_group, ui_actions, G_N_ELEMENTS (ui_actions), grid);
+
+	grid->priv->filter = NULL;
+	grid->priv->filter_window = NULL;
 }
 
 /**
@@ -333,6 +342,11 @@ gnome_db_raw_grid_dispose (GObject *object)
 			grid->priv->actions_group = NULL;
 		}
 
+		if (grid->priv->filter) 
+			gtk_widget_destroy (grid->priv->filter);
+		if (grid->priv->filter_window) 
+			gtk_widget_destroy (grid->priv->filter_window);
+
 		/* the private area itself */
 		g_free (grid->priv);
 		grid->priv = NULL;
@@ -355,7 +369,7 @@ gnome_db_raw_grid_set_property (GObject *object,
                 switch (param_id) {
 		case PROP_MODEL:
 			{
-				GdaDataModel *model = GDA_DATA_MODEL(g_value_get_object (value));
+				GdaDataModel *model = GDA_DATA_MODEL (g_value_get_object (value));
 				if (model)
 					g_return_if_fail (GDA_IS_DATA_MODEL (model));
 	
@@ -367,8 +381,7 @@ gnome_db_raw_grid_set_property (GObject *object,
 				grid->priv->proxy = gnome_db_data_store_get_proxy (grid->priv->store);
 				grid->priv->data_model = gda_data_proxy_get_proxied_model (grid->priv->proxy);
 				
-				gda_object_connect_destroy (grid->priv->proxy,
-								G_CALLBACK (proxy_destroyed_cb), grid);
+				g_object_ref (G_OBJECT (grid->priv->proxy));
 				g_signal_connect (grid->priv->proxy, "sample_changed",
 						  G_CALLBACK (proxy_sample_changed_cb), grid);
 				g_signal_connect (grid->priv->proxy, "row_updated",
@@ -380,7 +393,7 @@ gnome_db_raw_grid_set_property (GObject *object,
 				
 				g_signal_connect (grid->priv->iter, "public_data_changed",
 						  G_CALLBACK (paramlist_public_data_changed_cb), grid);
-				g_signal_connect (grid->priv->iter, "param_plugin_changed",
+				g_signal_connect (grid->priv->iter, "holder_plugin_changed",
 						  G_CALLBACK (paramlist_param_plugin_changed_cb), grid);
 
 				g_signal_connect (grid->priv->iter, "row_changed",
@@ -437,7 +450,7 @@ gnome_db_raw_grid_get_property (GObject *object,
         if (grid->priv) {
                 switch (param_id) {
 		case PROP_MODEL:
-			g_value_set_pointer (value, grid->priv->data_model);
+			g_value_set_object (value, grid->priv->data_model);
 			break;
 		case PROP_INFO_CELL_VISIBLE:
 			g_value_set_boolean(value, grid->priv->default_show_info_cell);
@@ -529,6 +542,7 @@ static void     data_cell_values_changed (GtkCellRenderer *renderer, const gchar
 					  GSList *new_values, GSList *all_new_values, GnomeDbRawGrid *grid);
 static void     data_cell_status_changed (GtkCellRenderer *renderer, const gchar *path, 
 					  GdaValueAttribute requested_action, GnomeDbRawGrid *grid);
+static void     treeview_column_clicked_cb (GtkTreeViewColumn *tree_column, GnomeDbRawGrid *grid);
 
 /*
  * Creates the GtkTreeView's columns, from the grid->priv->store GtkTreeModel
@@ -539,23 +553,21 @@ init_tree_view (GnomeDbRawGrid *grid)
 	gint i;
 	GtkTreeView *tree_view;
 	GSList *list;
-	GdaDict *dict;
 
 	tree_view = GTK_TREE_VIEW (grid);
 
 	/* Creation of the columns in the treeview, to fit the parameters in grid->priv->iter param list */
-	dict = gda_object_get_dict (GDA_OBJECT (grid->priv->iter));
-	list = GDA_PARAMETER_LIST (grid->priv->iter)->groups_list;
+	list = GDA_SET (grid->priv->iter)->groups_list;
 	i = 0;
 	while (list) {
-		GdaParameter *param;
-		GdaParameterListGroup *group;
+		GdaHolder *param;
+		GdaSetGroup *group;
 		GtkTreeViewColumn *column;
 		GtkCellRenderer *renderer;
 		ColumnData *column_data;
 		gboolean explicit_hide = FALSE;
 
-		group = GDA_PARAMETER_LIST_GROUP (list->data);
+		group = GDA_SET_GROUP (list->data);
 
 		/* update the list of columns data */
 		column_data = get_column_data (grid, group);
@@ -576,17 +588,18 @@ init_tree_view (GnomeDbRawGrid *grid)
 			
 			nodes = group->nodes;
 			while (nodes && nullok) {
-				if (gda_parameter_get_not_null (GDA_PARAMETER 
-								(GDA_PARAMETER_LIST_NODE (nodes->data)->param)))
+				if (gda_holder_get_not_null (GDA_HOLDER (GDA_SET_NODE (nodes->data)->holder)))
 					nullok = FALSE;
 				nodes = g_slist_next (nodes);
 			}
 			
 			/* determine title */
 			if (g_slist_length (group->nodes) == 1)
-				title = (gchar *) gda_object_get_name (GDA_OBJECT (GDA_PARAMETER_LIST_NODE (group->nodes->data)->param));
+				title = (gchar *) g_object_get_data (G_OBJECT (GDA_SET_NODE (group->nodes->data)->holder),
+								     "name");
 			else 
-				title = (gchar *) gda_object_get_name (GDA_OBJECT (group->nodes_source->data_model));
+				title = (gchar *) g_object_get_data (G_OBJECT (group->nodes_source->data_model),
+								     "name");
 
 			if (title)
 				title = replace_double_underscores (g_strdup (title));
@@ -596,7 +609,7 @@ init_tree_view (GnomeDbRawGrid *grid)
 
 			/* FIXME: if nullok is FALSE, then set the column title in bold */
 
-			renderer = gnome_db_data_cell_renderer_combo_new (GDA_PARAMETER_LIST (grid->priv->iter), group->nodes_source);
+			renderer = gnome_db_data_cell_renderer_combo_new (GDA_SET (grid->priv->iter), group->nodes_source);
 			column_data->data_cell = renderer;
 			gtk_tree_view_insert_column_with_data_func (tree_view, i, title, renderer,
 								    (GtkTreeCellDataFunc) cell_renderer_value_set_attributes, 
@@ -616,30 +629,30 @@ init_tree_view (GnomeDbRawGrid *grid)
 			gchar *plugin = NULL;
 			gchar *title;
 			
-			param = GDA_PARAMETER (GDA_PARAMETER_LIST_NODE (group->nodes->data)->param);
-			g_type = gda_parameter_get_g_type (param);
+			param = GDA_HOLDER (GDA_SET_NODE (group->nodes->data)->holder);
+			g_type = gda_holder_get_g_type (param);
 
-			title = (gchar *) gda_object_get_name (GDA_OBJECT (param));
+			g_object_get (G_OBJECT (param), "name", &title, NULL);
 			if (title && *title)
-				title = replace_double_underscores (g_strdup (title));
+				title = replace_double_underscores (title);
 			else
 				title = NULL;
 			if (!title)
 				title = g_strdup (_("No title"));
 			
-			g_object_get (G_OBJECT (param), "entry_plugin", &plugin, NULL);
-			renderer = gnome_db_util_new_cell_renderer (dict, g_type, plugin);
+			g_object_get (G_OBJECT (param), "plugin", &plugin, NULL);
+			renderer = gnome_db_util_new_cell_renderer (g_type, plugin);
 			column_data->data_cell = renderer;
 			if (! column_data->data_locked)
-				column_data->data_locked = GDA_PARAMETER_LIST_NODE (group->nodes->data)->hint & 
-					GDA_PARAMETER_LIST_PARAM_READ_ONLY;
+				column_data->data_locked = GDA_SET_NODE (group->nodes->data)->hint & 
+					GDA_SET_HOLDER_READ_ONLY;
 			gtk_tree_view_insert_column_with_data_func (tree_view, i, title, renderer,
 								    (GtkTreeCellDataFunc) cell_renderer_value_set_attributes, 
 								    grid, NULL);
 			column = gtk_tree_view_get_column (tree_view, i);
 			g_free (title);
 
-			if (GDA_PARAMETER_LIST_NODE (group->nodes->data)->hint & GDA_PARAMETER_LIST_PARAM_HIDE)
+			if (GDA_SET_NODE (group->nodes->data)->hint & GDA_SET_HOLDER_HIDE)
 				g_object_set_data (G_OBJECT (param), "_gnome_db_explicit_show", GINT_TO_POINTER (1));
 
 			if (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (param), "_gnome_db_explicit_show")) == 1)
@@ -675,9 +688,10 @@ init_tree_view (GnomeDbRawGrid *grid)
 		g_object_set_data (G_OBJECT (renderer), "group", group);
 
 		/* Sorting data */
-		/* gtk_tree_view_column_set_sort_column_id (column, i); */
-		/* gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (tree_model), */
-		/* i, (GtkTreeIterCompareFunc) tree_sortable_sort_values, node, NULL); */
+		gtk_tree_view_column_set_clickable (column, TRUE);
+		g_signal_connect (G_OBJECT (column), "clicked",
+				  G_CALLBACK (treeview_column_clicked_cb), grid);
+
 		list = g_slist_next (list);
 		i++;
 	}
@@ -685,24 +699,21 @@ init_tree_view (GnomeDbRawGrid *grid)
 	/*
 	 * Hiding some columns of the grid
 	 */
-	list = GDA_PARAMETER_LIST (grid->priv->iter)->nodes_list;
-	while (list) {
-		guint hint = GDA_PARAMETER_LIST_NODE (list->data)->hint;
+	for (list = GDA_SET (grid->priv->iter)->nodes_list; list; list = list->next) {
+		guint hint = GDA_SET_NODE (list->data)->hint;
 
-		if (hint & GDA_PARAMETER_LIST_PARAM_HIDE) {
+		if (hint & GDA_SET_HOLDER_HIDE) {
 			GtkTreeViewColumn *viewcol;
-			GdaParameterListGroup *group;
+			GdaSetGroup *group;
 			gint pos;
 
-			group = gda_parameter_list_find_group_for_param ((GdaParameterList *) grid->priv->iter, 
-									 GDA_PARAMETER_LIST_NODE (list->data)->param);
-			pos = g_slist_index (((GdaParameterList *) grid->priv->iter)->groups_list, group);
+			group = gda_set_get_group ((GdaSet *) grid->priv->iter, GDA_SET_NODE (list->data)->holder);
+			pos = g_slist_index (((GdaSet *) grid->priv->iter)->groups_list, group);
 			g_assert (pos >= 0);
 
 			viewcol = gtk_tree_view_get_column (GTK_TREE_VIEW (grid), pos);
 			gtk_tree_view_column_set_visible (viewcol, FALSE);
 		}
-		list = g_slist_next (list);
 	}
 }
 
@@ -716,7 +727,7 @@ cell_renderer_value_set_attributes (GtkTreeViewColumn *tree_column,
 				    GtkTreeModel *tree_model,
 				    GtkTreeIter *iter, GnomeDbRawGrid *grid)
 {
-	GdaParameterListGroup *group;
+	GdaSetGroup *group;
 	guint attributes;
 	gboolean to_be_deleted = FALSE;
 	ColumnData *column_data;
@@ -727,7 +738,7 @@ cell_renderer_value_set_attributes (GtkTreeViewColumn *tree_column,
 	if (group->nodes_source) {
 		/* parameters depending on a GdaDataModel */
 		GList *values = NULL;
-		GdaParameterListSource *source;
+		GdaSetSource *source;
 
 		source = g_object_get_data (G_OBJECT (tree_column), "source");
 
@@ -788,8 +799,8 @@ cell_renderer_value_set_attributes (GtkTreeViewColumn *tree_column,
 		offset = gda_data_model_get_n_columns (gda_data_proxy_get_proxied_model (grid->priv->proxy));
 
 		g_assert (g_slist_length (group->nodes) == 1);
-		col = gda_data_model_iter_get_column_for_param (grid->priv->iter, 
-								GDA_PARAMETER_LIST_NODE (group->nodes->data)->param);
+		col = g_slist_index (((GdaSet *)grid->priv->iter)->holders,
+				     GDA_SET_NODE (group->nodes->data)->holder);
 		gtk_tree_model_get (GTK_TREE_MODEL (grid->priv->store), iter, 
 				    DATA_STORE_COL_TO_DELETE, &to_be_deleted,
 				    DATA_STORE_COL_MODEL_ROW, &row,
@@ -817,7 +828,7 @@ cell_renderer_info_set_attributes (GtkTreeViewColumn *tree_column,
 				   GtkTreeModel *tree_model,
 				   GtkTreeIter *iter, GnomeDbRawGrid *grid)
 {
-	GdaParameterListGroup *group;
+	GdaSetGroup *group;
 	guint attributes;
 	gboolean to_be_deleted = FALSE;
 	ColumnData *column_data;
@@ -827,7 +838,7 @@ cell_renderer_info_set_attributes (GtkTreeViewColumn *tree_column,
 	
 	if (group->nodes_source) {
 		/* parameters depending on a GdaDataModel */
-		GdaParameterListSource *source;
+		GdaSetSource *source;
 
 		source = g_object_get_data (G_OBJECT (tree_column), "source");
 
@@ -851,8 +862,8 @@ cell_renderer_info_set_attributes (GtkTreeViewColumn *tree_column,
 		offset = gda_data_model_get_n_columns (gda_data_proxy_get_proxied_model (grid->priv->proxy));
 
 		g_assert (g_slist_length (group->nodes) == 1);
-		col = gda_data_model_iter_get_column_for_param (grid->priv->iter, 
-								GDA_PARAMETER_LIST_NODE (group->nodes->data)->param);
+		col = g_slist_index (((GdaSet *)grid->priv->iter)->holders,
+				     GDA_SET_NODE (group->nodes->data)->holder);
 		gtk_tree_model_get (GTK_TREE_MODEL (grid->priv->store), iter, 
 				    DATA_STORE_COL_TO_DELETE, &to_be_deleted,
 				    DATA_STORE_COL_MODEL_ROW, &row,
@@ -879,22 +890,17 @@ static void
 data_cell_value_changed (GtkCellRenderer *renderer, const gchar *path, const GValue *new_value, GnomeDbRawGrid *grid)
 {
 	GtkTreeIter iter;
-	GdaParameterListGroup *group;
+	GdaSetGroup *group;
 	
 	group = g_object_get_data (G_OBJECT (renderer), "group");
 	g_assert (g_slist_length (group->nodes) == 1);
 
 	if (set_iter_from_path (grid, path, &iter)) {
 		    gint col;
-		    col = gda_data_model_iter_get_column_for_param (grid->priv->iter, 
-								    GDA_PARAMETER_LIST_NODE (group->nodes->data)->param);
+		    col = g_slist_index (((GdaSet *)grid->priv->iter)->holders,
+					 GDA_SET_NODE (group->nodes->data)->holder);
 		    gnome_db_data_store_set_value (grid->priv->store, &iter, col, new_value);
 	}
-
-#ifdef debug_no
-	DEBUG_HEADER;
-	gda_object_dump (grid->priv->proxy, 5);
-#endif
 }
 
 
@@ -908,7 +914,7 @@ data_cell_values_changed (GtkCellRenderer *renderer, const gchar *path,
 			  GSList *new_values, GSList *all_new_values, GnomeDbRawGrid *grid)
 {
 	GtkTreeIter iter;
-	GdaParameterListGroup *group;
+	GdaSetGroup *group;
 	
 	group = g_object_get_data (G_OBJECT (renderer), "group");
 	g_assert (group->nodes_source);
@@ -919,11 +925,6 @@ data_cell_values_changed (GtkCellRenderer *renderer, const gchar *path,
 		/* the reason for not having any value is that the GnomeDbDataCellRendererCombo had no selected item */
 		return;
 	
-#ifdef debug_NO
-	DEBUG_HEADER;
-	gda_object_dump (grid->priv->proxy, 0);
-#endif
-
 	if (set_iter_from_path (grid, path, &iter)) {
 		GSList *list, *params;
 		gint col, proxy_row;
@@ -931,14 +932,12 @@ data_cell_values_changed (GtkCellRenderer *renderer, const gchar *path,
 		proxy_row = gnome_db_data_store_get_row_from_iter (grid->priv->store, &iter);
 
 		/* update the GnomeDbDataStore */
-		params = group->nodes;
-		list = new_values;
-		while (list) {
-			col = gda_data_model_iter_get_column_for_param (grid->priv->iter, 
-									GDA_PARAMETER_LIST_NODE (params->data)->param);
+		for (params = group->nodes, list = new_values; 
+		     list; 
+		     params = params->next, list = list->next) {
+			col = g_slist_index (((GdaSet *)grid->priv->iter)->holders,
+					     GDA_SET_NODE (params->data)->holder);
 			gnome_db_data_store_set_value (grid->priv->store, &iter, col, (GValue *)(list->data));
-			list = g_slist_next (list);
-			params = g_slist_next (params);
 		}
 		
 #ifdef PROXY_STORE_EXTRA_VALUES
@@ -955,11 +954,6 @@ data_cell_values_changed (GtkCellRenderer *renderer, const gchar *path,
 		}
 #endif
 	}
-
-#ifdef debug_NO
-	DEBUG_HEADER;
-	gda_object_dump (grid->priv->proxy, 0);
-#endif
 }
 
 static gboolean
@@ -980,6 +974,27 @@ set_iter_from_path (GnomeDbRawGrid *grid, const gchar *path, GtkTreeIter *iter)
 	return TRUE;
 }
 
+static void
+treeview_column_clicked_cb (GtkTreeViewColumn *tree_column, GnomeDbRawGrid *grid)
+{
+	GdaSetGroup *group;
+	GSList *nodes;
+
+	group = g_object_get_data (G_OBJECT (tree_column), "group");
+	g_assert (group);
+
+	for (nodes = group->nodes; nodes; nodes = nodes->next) {
+		GdaHolder *param = ((GdaSetNode*) nodes->data)->holder;
+		gint pos;
+		g_assert (param);
+		
+		pos = g_slist_index (GDA_SET (grid->priv->iter)->holders, param);
+		if (pos >= 0) {
+			gda_data_proxy_set_ordering_column (grid->priv->proxy, pos, NULL);
+			break;
+		}
+	}
+}
 
 /*
  * Actions
@@ -990,7 +1005,7 @@ data_cell_status_changed (GtkCellRenderer *renderer, const gchar *path, GdaValue
 {
 	GtkTreePath *treepath;
 	GtkTreeIter iter;
-	GdaParameterListGroup *group;
+	GdaSetGroup *group;
 	GtkTreeModel *tree_model;
 	gint col;
 	gint offset;
@@ -1013,14 +1028,13 @@ data_cell_status_changed (GtkCellRenderer *renderer, const gchar *path, GdaValue
 	if (group->nodes_source) {
 		/* parameters depending on a GdaDataModel */
 		gint proxy_row;
-		GSList *list = group->nodes;
+		GSList *list;
 		proxy_row = gnome_db_data_store_get_row_from_iter (grid->priv->store, &iter);
 
-		while (list) {
-			col = gda_data_model_iter_get_column_for_param (grid->priv->iter, 
-									GDA_PARAMETER_LIST_NODE (list->data)->param);
+		for (list = group->nodes; list; list = list->next) {
+			col = g_slist_index (((GdaSet *)grid->priv->iter)->holders,
+					     GDA_SET_NODE (list->data)->holder);
 			gnome_db_data_store_set_value (grid->priv->store, &iter, offset + col, attribute);
-			list = g_slist_next (list);
 		}
 
 #ifdef PROXY_STORE_EXTRA_VALUES
@@ -1054,16 +1068,11 @@ data_cell_status_changed (GtkCellRenderer *renderer, const gchar *path, GdaValue
 		gint col;
 
 		g_assert (g_slist_length (group->nodes) == 1);
-		col = gda_data_model_iter_get_column_for_param (grid->priv->iter, 
-								GDA_PARAMETER_LIST_NODE (group->nodes->data)->param);
+		col = g_slist_index (((GdaSet *)grid->priv->iter)->holders,
+				     GDA_SET_NODE (group->nodes->data)->holder);
 		gnome_db_data_store_set_value (grid->priv->store, &iter, offset + col, attribute);
 	}
 	gda_value_free (attribute);
-
-#ifdef debug_no
-	DEBUG_HEADER;
-	gda_object_dump (grid->priv->proxy, 0);
-#endif
 }
 
 static void
@@ -1211,6 +1220,155 @@ action_last_chunck_cb (GtkAction *action, GnomeDbRawGrid *grid)
 	gda_data_proxy_set_sample_start (grid->priv->proxy, G_MAXINT);
 }
 
+static void
+hide_filter_window (GnomeDbRawGrid *grid)
+{
+	gtk_widget_hide (grid->priv->filter_window);
+	gtk_grab_remove (grid->priv->filter_window);
+}
+
+static gboolean
+filter_event (GtkWidget *widget, GdkEventAny *event, GnomeDbRawGrid *grid)
+{
+	hide_filter_window (grid);
+	return TRUE;
+}
+
+static gboolean
+key_press_filter_event (GtkWidget *widget, GdkEventKey *event, GnomeDbRawGrid *grid)
+{
+	if (event->keyval == GDK_Escape ||
+	    event->keyval == GDK_Tab ||
+            event->keyval == GDK_KP_Tab ||
+            event->keyval == GDK_ISO_Left_Tab) {
+		hide_filter_window (grid);
+		return TRUE;
+	}
+	return FALSE;
+}
+
+static void
+filter_position_func (GtkWidget *widget,
+		      GtkWidget *search_dialog,
+		      gpointer   user_data)
+{
+	gint x, y;
+	gint tree_x, tree_y;
+	gint tree_width, tree_height;
+	GdkWindow *window = GTK_WIDGET (widget)->window;
+	GdkScreen *screen = gdk_drawable_get_screen (window);
+	GtkRequisition requisition;
+	gint monitor_num;
+	GdkRectangle monitor;
+
+	monitor_num = gdk_screen_get_monitor_at_window (screen, window);
+	gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
+  
+	gtk_widget_realize (search_dialog);
+
+	gdk_window_get_origin (window, &tree_x, &tree_y);
+	gdk_drawable_get_size (window,
+			       &tree_width,
+			       &tree_height);
+	gtk_widget_size_request (search_dialog, &requisition);
+
+	if (tree_x + tree_width > gdk_screen_get_width (screen))
+		x = gdk_screen_get_width (screen) - requisition.width;
+	else if (tree_x + tree_width - requisition.width < 0)
+		x = 0;
+	else
+		x = tree_x + tree_width - requisition.width;
+
+	if (tree_y + tree_height + requisition.height > gdk_screen_get_height (screen))
+		y = gdk_screen_get_height (screen) - requisition.height;
+	else if (tree_y + tree_height < 0) /* isn't really possible ... */
+		y = 0;
+	else
+		y = tree_y + tree_height;
+
+	gtk_window_move (GTK_WINDOW (search_dialog), x, y);
+}
+
+static gboolean
+popup_grab_on_window (GdkWindow  *window,
+                      guint32     activate_time) 
+{
+        if (gdk_pointer_grab (window, TRUE,
+                              GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK,
+                              NULL, NULL,
+                              activate_time) == 0) {
+
+                if (gdk_keyboard_grab (window, TRUE, activate_time) == 0) 
+                        return TRUE;
+		else {
+                        gdk_pointer_ungrab (activate_time);
+                        return FALSE;
+                }
+        }
+
+        return FALSE;
+}
+
+static void
+action_filter_cb (GtkAction *action, GnomeDbRawGrid *grid)
+{
+	GtkWidget *toplevel;
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (grid));
+	
+	if (!grid->priv->filter_window) {
+		/* create filter window */
+		GtkWidget *frame, *vbox;
+	       
+		grid->priv->filter_window = gtk_window_new (GTK_WINDOW_POPUP);
+
+		gtk_widget_set_events (grid->priv->filter_window,
+				       gtk_widget_get_events (grid->priv->filter_window) | GDK_KEY_PRESS_MASK);
+
+		if (GTK_WIDGET_TOPLEVEL (toplevel) && GTK_WINDOW (toplevel)->group)
+			gtk_window_group_add_window (GTK_WINDOW (toplevel)->group,
+						     GTK_WINDOW (grid->priv->filter_window));
+		
+		g_signal_connect (grid->priv->filter_window, "delete_event",
+				  G_CALLBACK (filter_event), grid);
+		g_signal_connect (grid->priv->filter_window, "button_press_event",
+				  G_CALLBACK (filter_event), grid);
+		g_signal_connect (grid->priv->filter_window, "key_press_event",
+				  G_CALLBACK (key_press_filter_event), grid);
+		frame = gtk_frame_new (NULL);
+		gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
+		gtk_widget_show (frame);
+		gtk_container_add (GTK_CONTAINER (grid->priv->filter_window), frame);
+		
+		vbox = gtk_vbox_new (FALSE, 0);
+		gtk_widget_show (vbox);
+		gtk_container_add (GTK_CONTAINER (frame), vbox);
+		gtk_container_set_border_width (GTK_CONTAINER (vbox), 3);
+		
+		/* add real filter widget */
+		if (! grid->priv->filter) {
+			grid->priv->filter = gnome_db_data_widget_filter_new (GNOME_DB_DATA_WIDGET (grid));
+			gtk_widget_show (grid->priv->filter);
+		}
+		gtk_container_add (GTK_CONTAINER (vbox), grid->priv->filter);
+	}
+	else if (GTK_WIDGET_TOPLEVEL (toplevel)) {
+		if (GTK_WINDOW (toplevel)->group)
+			gtk_window_group_add_window (GTK_WINDOW (toplevel)->group,
+						     GTK_WINDOW (grid->priv->filter_window));
+		else if (GTK_WINDOW (grid->priv->filter_window)->group)
+			gtk_window_group_remove_window (GTK_WINDOW (grid->priv->filter_window)->group,
+							GTK_WINDOW (grid->priv->filter_window));
+	}
+
+	/* move the filter window to a correct location */
+	/* FIXME: let the user specify the position function like GtkTreeView -> search_position_func() */
+	gtk_grab_add (grid->priv->filter_window);
+	filter_position_func (GTK_WIDGET (grid), grid->priv->filter_window, NULL);
+	gtk_widget_show (grid->priv->filter_window);
+	popup_grab_on_window (grid->priv->filter_window->window,
+			      gtk_get_current_event_time ());	
+}
+
 /*
  * Catch any event in the GtkTreeView widget
  */
@@ -1291,6 +1449,8 @@ static void menu_select_all_cb (GtkWidget *widget, GnomeDbRawGrid *grid);
 static void menu_unselect_all_cb (GtkWidget *widget, GnomeDbRawGrid *grid);
 static void menu_show_columns_cb (GtkWidget *widget, GnomeDbRawGrid *grid);
 static void menu_save_as_cb (GtkWidget *widget, GnomeDbRawGrid *grid);
+static void menu_set_filter_cb (GtkWidget *widget, GnomeDbRawGrid *grid);
+static void menu_unset_filter_cb (GtkWidget *widget, GnomeDbRawGrid *grid);
 
 static gint
 tree_view_popup_button_pressed_cb (GtkWidget *widget, GdkEventButton *event, GnomeDbRawGrid *grid)
@@ -1323,6 +1483,13 @@ tree_view_popup_button_pressed_cb (GtkWidget *widget, GdkEventButton *event, Gno
 						       gtk_tree_view_get_headers_visible (tree_view),
 						       G_CALLBACK (menu_show_columns_cb), grid));
 
+	gtk_menu_append (GTK_MENU (menu),
+				 gnome_db_new_menu_item (_("_Set filter"), FALSE,
+							 G_CALLBACK (menu_set_filter_cb), grid));
+	gtk_menu_append (GTK_MENU (menu),
+				 gnome_db_new_menu_item (_("_Unset filter"), FALSE,
+							 G_CALLBACK (menu_unset_filter_cb), grid));
+
 	if (sel_mode != GTK_SELECTION_NONE) {
 		gtk_menu_append (GTK_MENU (menu), gtk_separator_menu_item_new ());
 		gtk_menu_append (GTK_MENU (menu), gnome_db_new_menu_item (GTK_STOCK_SAVE_AS, TRUE,
@@ -1493,8 +1660,8 @@ save_as_response_cb (GtkDialog *dialog, guint response_id, GnomeDbRawGrid *grid)
 		GList *columns, *list;
 		gint *cols, nb_cols;
 		gint *rows = NULL, nb_rows = 0;
-		GdaParameter *param;
-		GdaParameterList *paramlist;
+		GdaHolder *param;
+		GdaSet *paramlist;
 
 		types = g_object_get_data (G_OBJECT (dialog), "types");
 		filename = g_object_get_data (G_OBJECT (dialog), "filename");
@@ -1508,17 +1675,13 @@ save_as_response_cb (GtkDialog *dialog, guint response_id, GnomeDbRawGrid *grid)
 		list = columns;
 		while (list) {
 			if (gtk_tree_view_column_get_visible (GTK_TREE_VIEW_COLUMN (list->data))) {
-				GdaParameterListGroup *group;
+				GdaSetGroup *group;
 				GSList *params;
 
 				group = g_object_get_data (G_OBJECT (list->data), "group");
-				params = group->nodes;
-				while (params) {
-					cols [nb_cols] = gda_data_model_iter_get_column_for_param (grid->priv->iter, 
-									GDA_PARAMETER_LIST_NODE (params->data)->param);
-					nb_cols ++;
-					params = g_slist_next (params);
-				}
+				for (params = group->nodes; params; nb_cols ++, params = params->next) 
+					cols [nb_cols] = g_slist_index (((GdaSet *)grid->priv->iter)->holders,
+									GDA_SET_NODE (params->data)->holder);
 			}
 			list = g_list_next (list);
 		}
@@ -1549,9 +1712,9 @@ save_as_response_cb (GtkDialog *dialog, guint response_id, GnomeDbRawGrid *grid)
 		grid->priv->export_type = export_type;
 		switch (export_type) {
 		case 0:
-			param = gda_parameter_new_string ("SEPARATOR", "\t");
-			paramlist = gda_parameter_list_new (NULL);
-			gda_parameter_list_add_param (paramlist, param);
+			param = gda_holder_new_string ("SEPARATOR", "\t");
+			paramlist = gda_set_new (NULL);
+			gda_set_add_holder (paramlist, param);
 			g_object_unref (param);
 			body = gda_data_model_export_to_string (GDA_DATA_MODEL (grid->priv->data_model), 
 								GDA_DATA_MODEL_IO_TEXT_SEPARATED, 
@@ -1559,9 +1722,9 @@ save_as_response_cb (GtkDialog *dialog, guint response_id, GnomeDbRawGrid *grid)
 			g_object_unref (paramlist);
 			break;
 		case 1:
-			param = gda_parameter_new_string ("SEPARATOR", ",");
-			paramlist = gda_parameter_list_new (NULL);
-			gda_parameter_list_add_param (paramlist, param);
+			param = gda_holder_new_string ("SEPARATOR", ",");
+			paramlist = gda_set_new (NULL);
+			gda_set_add_holder (paramlist, param);
 			g_object_unref (param);
 			body = gda_data_model_export_to_string (GDA_DATA_MODEL (grid->priv->data_model), 
 								GDA_DATA_MODEL_IO_TEXT_SEPARATED, 
@@ -1570,11 +1733,11 @@ save_as_response_cb (GtkDialog *dialog, guint response_id, GnomeDbRawGrid *grid)
 			break;
 		case 2:
 			param = NULL;
-			body = (gchar *) gda_object_get_name (GDA_OBJECT (grid->priv->data_model));
+			body = (gchar *) g_object_get_data (G_OBJECT (grid->priv->data_model), "name");
 			if (body)
-				param = gda_parameter_new_string ("NAME", body);
-			paramlist = gda_parameter_list_new (NULL);
-			gda_parameter_list_add_param (paramlist, param);
+				param = gda_holder_new_string ("NAME", body);
+			paramlist = gda_set_new (NULL);
+			gda_set_add_holder (paramlist, param);
 			g_object_unref (param);
 			body = gda_data_model_export_to_string (GDA_DATA_MODEL (grid->priv->data_model), 
 								GDA_DATA_MODEL_IO_DATA_ARRAY_XML, 
@@ -1601,7 +1764,7 @@ save_as_response_cb (GtkDialog *dialog, guint response_id, GnomeDbRawGrid *grid)
 					}
 				}
 
-				if (! gda_file_save (path, body, strlen (body))) {
+				if (! g_file_set_contents (path, body, strlen (body), NULL)) {
 					gnome_db_show_error (NULL, _("Could not save file %s"), path);
 					g_free (body);
 					g_free (path);
@@ -1659,6 +1822,18 @@ confirm_file_overwrite (GtkWindow *parent, const gchar *path)
 	return yes;
 }
 
+static void
+menu_set_filter_cb (GtkWidget *widget, GnomeDbRawGrid *grid)
+{
+	action_filter_cb (NULL, grid);
+}
+
+static void
+menu_unset_filter_cb (GtkWidget *widget, GnomeDbRawGrid *grid)
+{
+	gda_data_proxy_set_filter_expr (grid->priv->proxy, NULL, NULL);
+}
+
 
 static void
 tree_view_row_activated_cb (GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, GnomeDbRawGrid *grid)
@@ -1737,7 +1912,7 @@ tree_view_selection_changed_cb (GtkTreeSelection *selection, GnomeDbRawGrid *gri
 }
 
 static ColumnData *
-get_column_data (GnomeDbRawGrid *grid, GdaParameterListGroup *group)
+get_column_data (GnomeDbRawGrid *grid, GdaSetGroup *group)
 {
 	ColumnData *retval = NULL;
 	GSList *list = grid->priv->columns_data;
@@ -1801,9 +1976,9 @@ static void
 gnome_db_raw_grid_set_column_editable (GnomeDbDataWidget *iface, gint column, gboolean editable)
 {
 	GnomeDbRawGrid *grid;
-	GdaParameter *param;
+	GdaHolder *param;
 	ColumnData *column_data;
-	GdaParameterListGroup *group;
+	GdaSetGroup *group;
 
 	g_return_if_fail (iface && GNOME_DB_IS_RAW_GRID (iface));
 	grid = GNOME_DB_RAW_GRID (iface);
@@ -1812,10 +1987,10 @@ gnome_db_raw_grid_set_column_editable (GnomeDbDataWidget *iface, gint column, gb
 	if (grid->priv->data_model) {
 		editable = editable && !gda_data_proxy_is_read_only (grid->priv->proxy);
 
-		param = gda_data_model_iter_get_param_for_column (grid->priv->iter, column);
+		param = gda_data_model_iter_get_holder_for_field (grid->priv->iter, column);
 		g_return_if_fail (param);
 
-		group = gda_parameter_list_find_group_for_param (GDA_PARAMETER_LIST (grid->priv->iter), param);
+		group = gda_set_get_group (GDA_SET (grid->priv->iter), param);
 		g_return_if_fail (group);
 
 		column_data = get_column_data (grid, group);
@@ -1838,15 +2013,15 @@ gnome_db_raw_grid_show_column_actions (GnomeDbDataWidget *iface, gint column, gb
 	g_return_if_fail (grid->priv);
 
 	if (column >= 0) {
-		GdaParameter *param;
-		GdaParameterListGroup *group;
+		GdaHolder *param;
+		GdaSetGroup *group;
 		ColumnData *cdata;
 
 		/* setting applies only to the @column column */
-		param = gda_data_model_iter_get_param_for_column (grid->priv->iter, column);
+		param = gda_data_model_iter_get_holder_for_field (grid->priv->iter, column);
 		g_return_if_fail (param);
 		
-		group = gda_parameter_list_find_group_for_param (GDA_PARAMETER_LIST (grid->priv->iter), param);
+		group = gda_set_get_group (GDA_SET (grid->priv->iter), param);
 		g_return_if_fail (group);
 
 		cdata = get_column_data (grid, group);
@@ -1877,18 +2052,18 @@ gnome_db_raw_grid_col_set_show (GnomeDbDataWidget *iface, gint column, gboolean
 	GnomeDbRawGrid *grid;
 	gint pos = -1;
 	GtkTreeViewColumn *viewcol;
-	GdaParameterListGroup *group;
-	GdaParameter *param;
+	GdaSetGroup *group;
+	GdaHolder *param;
 
 	g_return_if_fail (iface && GNOME_DB_IS_RAW_GRID (iface));
 	grid = GNOME_DB_RAW_GRID (iface);
 	g_return_if_fail (grid->priv);
 
-	param = gda_data_model_iter_get_param_for_column (grid->priv->iter, column);
+	param = gda_data_model_iter_get_holder_for_field (grid->priv->iter, column);
 	g_return_if_fail (param);
 
-	group = gda_parameter_list_find_group_for_param ((GdaParameterList *)grid->priv->iter, param);	
-	pos = g_slist_index (((GdaParameterList *)grid->priv->iter)->groups_list, group);	
+	group = gda_set_get_group ((GdaSet *)grid->priv->iter, param);	
+	pos = g_slist_index (((GdaSet *)grid->priv->iter)->groups_list, group);	
 	g_assert (pos >= 0);
 
 	viewcol = gtk_tree_view_get_column (GTK_TREE_VIEW (grid), pos);
@@ -1938,7 +2113,7 @@ gnome_db_raw_grid_widget_get_gda_model (GnomeDbDataWidget *iface)
 }
 
 static void
-paramlist_public_data_changed_cb (GdaParameterList *paramlist, GnomeDbRawGrid *grid)
+paramlist_public_data_changed_cb (GdaSet *paramlist, GnomeDbRawGrid *grid)
 {
 	GList *columns, *list;
 
@@ -1968,7 +2143,7 @@ paramlist_public_data_changed_cb (GdaParameterList *paramlist, GnomeDbRawGrid *g
 }
 
 static void
-paramlist_param_plugin_changed_cb (GdaParameterList *paramlist, GdaParameter *param, GnomeDbRawGrid *grid)
+paramlist_param_plugin_changed_cb (GdaSet *paramlist, GdaHolder *param, GnomeDbRawGrid *grid)
 {
 	/* TODO: be more specific and change only the cell renderer corresponding to @param */
 	paramlist_public_data_changed_cb (paramlist, grid);
@@ -2045,6 +2220,12 @@ proxy_row_updated_cb (GdaDataProxy *proxy, gint proxy_row, GnomeDbRawGrid *grid)
 			/* write back the current row */
 			if (gda_data_proxy_row_has_changed (grid->priv->proxy, row)) {
 				GError *error = NULL;
+
+				/* If the write fails, the proxy sets the fields back to was they were. We do not want
+				 * te get notified about this because we would the try again to write to the database,
+				 * which would fail again, and so on. */
+				g_signal_handlers_block_by_func(G_OBJECT(proxy), G_CALLBACK(proxy_row_updated_cb), grid);
+
 				if (!gda_data_proxy_apply_row_changes (grid->priv->proxy, row, &error)) {
 					gboolean discard;
 					discard = gnome_db_utility_display_error_with_keep_or_discard_choice ((GnomeDbDataWidget *) grid, 
@@ -2053,6 +2234,8 @@ proxy_row_updated_cb (GdaDataProxy *proxy, gint proxy_row, GnomeDbRawGrid *grid)
 						gda_data_proxy_cancel_row_changes (grid->priv->proxy, row, -1);
 					g_error_free (error);
 				}
+
+				g_signal_handlers_unblock_by_func(G_OBJECT(proxy), G_CALLBACK(proxy_row_updated_cb), grid);
 			}
 		}
 	}
@@ -2066,17 +2249,6 @@ proxy_reset_cb (GdaDataProxy *proxy, GnomeDbRawGrid *grid)
 	g_object_unref (G_OBJECT (proxy));
 }
 
-static void
-proxy_destroyed_cb (GdaDataProxy *proxy, GnomeDbRawGrid *grid)
-{
-	g_assert (proxy == grid->priv->proxy);
-	g_signal_handlers_disconnect_by_func (proxy, G_CALLBACK (proxy_destroyed_cb), grid);
-	g_signal_handlers_disconnect_by_func (proxy, G_CALLBACK (proxy_sample_changed_cb), grid);
-	g_signal_handlers_disconnect_by_func (proxy, G_CALLBACK (proxy_row_updated_cb), grid);
-	g_signal_handlers_disconnect_by_func (proxy, G_CALLBACK (proxy_reset_cb), grid);
-	grid->priv->proxy = NULL;
-}
-
 static void
 gnome_db_raw_grid_widget_set_gda_model (GnomeDbDataWidget *iface, GdaDataModel *model)
 {
@@ -2130,8 +2302,13 @@ gnome_db_raw_grid_clean (GnomeDbRawGrid *grid)
 	}
 	
 	/* proxy */
-	if (grid->priv->proxy)
-		proxy_destroyed_cb (grid->priv->proxy, grid);
+	if (grid->priv->proxy) {
+		g_signal_handlers_disconnect_by_func (grid->priv->proxy, G_CALLBACK (proxy_sample_changed_cb), grid);
+		g_signal_handlers_disconnect_by_func (grid->priv->proxy, G_CALLBACK (proxy_row_updated_cb), grid);
+		g_signal_handlers_disconnect_by_func (grid->priv->proxy, G_CALLBACK (proxy_reset_cb), grid);
+		g_object_unref (grid->priv->proxy);
+		grid->priv->proxy = NULL;
+	}
 
 	/* store */
 	if (grid->priv->store) {
