@@ -1,6 +1,6 @@
 /* gnome-db-transaction-status.c
  *
- * Copyright (C) 2002 - 2007 Vivien Malerba
+ * Copyright (C) 2002 - 2008 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,13 +28,13 @@ static void gnome_db_transaction_status_init (GnomeDbTransactionStatus *wid);
 static void gnome_db_transaction_status_dispose (GObject *object);
 
 static void gnome_db_transaction_status_set_property (GObject *object,
-                                              guint param_id,
-                                              const GValue *value,
-                                              GParamSpec *pspec);
+						      guint param_id,
+						      const GValue *value,
+						      GParamSpec *pspec);
 static void gnome_db_transaction_status_get_property (GObject *object,
-                                              guint param_id,
-                                              GValue *value,
-                                              GParamSpec *pspec);
+						      guint param_id,
+						      GValue *value,
+						      GParamSpec *pspec);
 
 static void gnome_db_transaction_status_refresh (GnomeDbTransactionStatus *status);
 static void gnome_db_transaction_status_clean (GnomeDbTransactionStatus *status);
@@ -126,13 +126,9 @@ gnome_db_transaction_status_init (GnomeDbTransactionStatus * wid)
 GtkWidget *
 gnome_db_transaction_status_new (GdaConnection *cnc)
 {
-	GObject *obj;
-	
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+	g_return_val_if_fail (!cnc || GDA_IS_CONNECTION (cnc), NULL);
 
-	obj = g_object_new (GNOME_DB_TYPE_TRANSACTION_STATUS, "connection", cnc, NULL);
-
-	return (GtkWidget *) obj;
+	return (GtkWidget *) g_object_new (GNOME_DB_TYPE_TRANSACTION_STATUS, "connection", cnc, NULL);
 }
 
 static void
@@ -250,7 +246,7 @@ transaction_status_get_depth_n_length (GdaTransactionStatus *status, gint *depth
 		switch (ev->type) {
 		case GDA_TRANSACTION_STATUS_EVENT_SAVEPOINT:
 		case GDA_TRANSACTION_STATUS_EVENT_SQL:
-			*length++;
+			(*length)++;
 			break;
 		case GDA_TRANSACTION_STATUS_EVENT_SUB_TRANSACTION: {
 			gint ldepth, llength;
